#! /usr/bin/python3

import html5_parser
import urllib.parse
import urllib.request
import json
import psycopg2
import ssl

dsn = 'dbname=investment-game user=investment-game'

asset_isin_insert_query = """
        INSERT INTO stocks_asset (
            isin,
            name,
            ticker
        ) VALUES (%s, %s, %s)
        RETURNING id
    """


def main():
    pg = psycopg2.connect(dsn)

    url = "https://live.euronext.com/en/ajax/getIndexCompositionFull/NL0000000107-XAMS"
    ctx = ssl.create_default_context()
    ctx.set_ciphers('DEFAULT@SECLEVEL=1')
    req = urllib.request.Request(
        url,
        headers={
            'User-Agent':
                'Mozilla/5.0 (X11; U; Linux i686) Gecko/20071127 Firefox/2.0.0.11',
        },
    )
    with urllib.request.urlopen(req, context=ctx) as fh:
        root = html5_parser.parse(fh.read())

    # with open('euronext.html') as fh:
    #     root = html5_parser.parse(fh.read())

    isins = {
        e.xpath('string(td[2])')
        for e in root.xpath('/html/body/table/tbody/tr')
    }

    if not isins:
        raise Exception("no isins found")

    with pg.cursor() as cursor:
        cursor.execute(
            "SELECT id, isin FROM stocks_asset WHERE isin IN %s",
            (tuple(isins),),
        )

        euronext_assets = set()

        for pk, isin in cursor.fetchall():
            euronext_assets.add(pk)
            isins.discard(isin)

        # Get ticker by isin
        for isin in isins:
            query = urllib.parse.urlencode({
                'q': isin,
                'token': 'c1q58oaad3ica9br00n0',
            })
            url = f'https://finnhub.io/api/v1/search?{query}'
            # print(url)
            with urllib.request.urlopen(url) as fh:
                finnhub_json = json.load(fh)

            if finnhub_json['count'] != 1 or len(finnhub_json['result']) != 1:
                raise Exception(
                    f"Expected exactly one result from {url}, "
                    f"but got {json.dumps(finnhub_json)}"
                )

            # print(repr(finnhub_json))

            result, = finnhub_json['result']

            cursor.execute(asset_isin_insert_query, (
                isin,
                result['description'],
                result['symbol'],
            ))

            (pk,), = cursor.fetchall()
            euronext_assets.add(pk)

        existing_assets = set()
        cursor.execute(
            "SELECT id FROM stocks_index WHERE name = 'AEX'"
        )
        # This isn't really a loop. Only get the first result of the iterator.
        for index_pk, in cursor.fetchall():
            cursor.execute(
                "SELECT asset_id FROM stocks_index_assets WHERE index_id = %s",
                (index_pk,)
            )
            for pk, in cursor.fetchall():
                existing_assets.add(pk)
            break
        else:
            cursor.execute(
                "INSERT INTO stocks_index (name) VALUES ('AEX') RETURNING id"
            )
            (index_pk,), = cursor.fetchall()

        for pk in existing_assets - euronext_assets:
            cursor.execute(
                """
                DELETE FROM stocks_index_assets
                WHERE index_id = %s AND asset_id = %s
                """,
                (index_pk, pk)
            )

        for pk in euronext_assets - existing_assets:
            cursor.execute(
                """
                INSERT INTO stocks_index_assets (index_id, asset_id)
                VALUES (%s, %s)
                """,
                (index_pk, pk)
            )

        pg.commit()


if __name__ == '__main__':
    main()
